# last modified 2017-02-10 by J. Fox

av.plot <- function (...) {
	.Defunct("avPlot", package="car")
}

av.plots <- function (...) {
	.Defunct("avPlots", package="car")
}

box.cox <- function (...) {
	.Defunct("bcPower", package="car")
}

bc <- function (...) {
	.Defunct("bcPower", package="car")
}

box.cox.powers <- function (...) {
	.Defunct("powerTransform", package="car")
}

box.cox.var <- function (...) {
	.Defunct("boxCoxVariable", package="car")
}

box.tidwell <- function (...) {
	.Defunct("boxTidwell", package="car")
}

ceres.plot <- function (...) {
	.Defunct("ceresPlot", package="car")
}

ceres.plots <- function (...) {
	.Defunct("ceresPlots", package="car")
}

confidence.ellipse <- function (...) {
	.Defunct("confidenceEllipse", package="car")
}

cookd <- function (...) {
	.Defunct("cooks.distance", package="stats")
}

cr.plot <- function (...) {
	.Defunct("crPlot", package="car")
}

cr.plots <- function (...) {
	.Defunct("crPlots", package="car")
}

data.ellipse <- function (...) {
	.Defunct("dataEllipse", package="car")
}

durbin.watson <- function (...) {
	.Defunct("durbinWatsonTest", package="car")
}

levene.test <- function (...) {
	.Defunct("leveneTest", package="car")
}

leverage.plot <- function (...) {
	.Defunct("leveragePlot", package="car")
}

leverage.plots <- function (...) {
	.Defunct("leveragePlots", package="car")
}

linear.hypothesis <- function (...) {
	.Defunct("linearHypothesis", package="car")
}

outlier.test <- function (...) {
	.Defunct("outlierTest", package="car")
}

ncv.test <- function (...) {
	.Defunct("ncvTest", package="car")
}

qq.plot <- function (...) {
	.Defunct("qqPlot", package="car")
}

scatterplot.matrix <- function (...) {
	.Defunct("scatterplotMatrix", package="car")
}

skewPower <- function (...) {
  .Defunct("bcnPower", package="car")
}

spread.level.plot <- function (...) {
	.Defunct("spreadLevelPlot", package="car")
}
